#!/usr/bin/env python3

import argparse
import asyncio
import json

from myPyllant.services import get_systems, get_session, get_devices_by_system
from myPyllant.session import login

parser = argparse.ArgumentParser(description="Export data from myVaillant API.")
parser.add_argument("user", help="Username (email address) for the myVaillant app")
parser.add_argument("password", help="Password for the myVaillant app")
parser.add_argument(
    "-d",
    "--data",
    action="store_true",
    help="Export historical device data (energy useage, etc.)",
)


async def main(user, password, data=False):
    try:
        await login(user, password)
        async for system in get_systems():
            if data:
                data_list = [
                    d.dict()
                    async for d in get_devices_by_system(system, get_buckets=True)
                ]
                print(
                    json.dumps(
                        data_list,
                        indent=2,
                        default=str,
                    )
                )
            else:
                print(json.dumps(system.dict(), indent=2, default=str))
    finally:
        await get_session().close()


if __name__ == "__main__":
    args = parser.parse_args()
    asyncio.run(main(args.user, args.password, args.data))



import argparse
import asyncio
from datetime import datetime, timedelta

from myPyllant.session import login, get_session
from myPyllant.services import (
    get_systems,
    set_holiday,
    cancel_holiday,
    boost_domestic_hot_water,
    cancel_quick_veto_zone_temperature,
    cancel_hot_water_boost,
    set_domestic_hot_water_temperature,
    set_set_back_temperature,
    quick_veto_zone_temperature,
)


parser = argparse.ArgumentParser(description="Export data from myVaillant API   .")
parser.add_argument("user", help="Username (email address) for the myVaillant app")
parser.add_argument("password", help="Password for the myVaillant app")


async def main(user, password):
    try:
        await login(user, password)
        async for system in get_systems():
            print(await set_holiday(system, datetime.now()))
            print(
                await set_holiday(
                    system, datetime.now(), datetime.now() + timedelta(days=1)
                )
            )
            print(await cancel_holiday(system))
            print(await boost_domestic_hot_water(system.domestic_hot_water[0]))
            print(await cancel_hot_water_boost(system.domestic_hot_water[0]))
            print(
                await set_domestic_hot_water_temperature(
                    system.domestic_hot_water[0], 46
                )
            )
            print(await set_set_back_temperature(system.zones[0], 15.5))
            print(await quick_veto_zone_temperature(system.zones[0], 21, 5))
            print(await cancel_quick_veto_zone_temperature(system.zones[0]))
    finally:
        await get_session().close()


if __name__ == "__main__":
    args = parser.parse_args()
    asyncio.run(main(args.user, args.password))



import aiohttp
from urllib.parse import urlencode, urlparse, parse_qs

from myPyllant.utils import generate_code, random_string


_aiohttp_session = None
_access_token = None
login_headers = {
    "accept": "application/json",
    "content-type": "application/json",
    "user-agent": "myVAILLANT/11835 CFNetwork/1240.0.4 Darwin/20.6.0",
    "x-okta-user-agent-extended": "okta-auth-js/5.4.1 okta-react-native/2.7.0 react-native/>=0.70.1 ios/14.8 nodejs/undefined",
    "accept-language": "de-de",
}
login_url = "https://vaillant-prod.okta.com/api/v1/authn"


def get_session():
    global _aiohttp_session
    if not _aiohttp_session:
        jar = aiohttp.CookieJar()
        _aiohttp_session = aiohttp.ClientSession(cookie_jar=jar)
    return _aiohttp_session


def set_access_token(access_token):
    global _access_token
    _access_token = access_token


def get_access_token():
    if not _access_token:
        raise Exception("No access token set, did you call login()?")
    return _access_token


async def login(username, password):
    session = get_session()
    code_verifier, code_challenge = generate_code()

    login_payload = {
        "username": username,
        "password": password,
    }
    async with session.post(
        login_url, json=login_payload, headers=login_headers
    ) as resp:
        response = await resp.json()
        if "errorSummary" in response:
            raise Exception(response["errorSummary"])
        session_token = response["sessionToken"]

    nonce = random_string(43)
    state = random_string(43)
    authorize_querystring = {
        "nonce": nonce,
        "sessionToken": session_token,
        "response_type": "code",
        "code_challenge_method": "S256",
        "scope": "openid profile offline_access",
        "code_challenge": code_challenge,
        "redirect_uri": "com.okta.vaillant-prod:/callback",
        "client_id": "0oarllti4egHi7Nwx4x6",
        "state": state,
    }
    authorize_headers = {
        "accept": "application/json",
        "content-type": "application/x-www-form-urlencoded",
        "user-agent": "myVAILLANT/11835 CFNetwork/1240.0.4 Darwin/20.6.0",
        "accept-language": "de-de",
    }
    authorize_url = (
        "https://vaillant-prod.okta.com/oauth2/default/v1/authorize?"
        + urlencode(authorize_querystring)
    )
    async with session.get(
        authorize_url, headers=authorize_headers, allow_redirects=False
    ) as resp:
        await resp.text()
        parsed_url = urlparse(resp.headers["Location"])
        code = parse_qs(parsed_url.query)["code"]

    token_payload = {
        "code": code,
        "code_verifier": code_verifier,
        "redirect_uri": "com.okta.vaillant-prod:/callback",
        "client_id": "0oarllti4egHi7Nwx4x6",
        "grant_type": "authorization_code",
    }
    token_headers = {
        "accept": "*/*",
        "content-type": "application/x-www-form-urlencoded; charset=UTF-8",
        "user-agent": "okta-react-native/2.7.0 okta-oidc-ios/3.11.2 react-native/>=0.70.1 ios/14.8",
        "accept-language": "de-de",
    }
    token_url = "https://vaillant-prod.okta.com/oauth2/default/v1/token"
    async with session.post(
        token_url, data=token_payload, headers=token_headers
    ) as resp:
        session_data = await resp.json()
        set_access_token(session_data["access_token"])


def get_authorized_headers():
    return {
        "Authorization": "Bearer " + get_access_token(),
        "x-app-identifier": "VAILLANT",
        "Accept-Language": "de-de",
        "Accept": "application/json, text/plain, */*",
        "x-client-locale": "de-DE",
        "x-idm-identifier": "OKTA",
        "ocp-apim-subscription-key": "1e0a2f3511fb4c5bbb1c7f9fedd20b1c",
        "User-Agent": "myVAILLANT/11835 CFNetwork/1240.0.4 Darwin/20.6.0",
        "Connection": "keep-alive",
    }



from typing import Any, Dict, List
import datetime

from pydantic import BaseModel


class Zone(BaseModel):
    system_id: str
    name: str
    index: int
    active: bool
    current_room_temperature: float
    current_special_function: str
    desired_room_temperature_setpoint: float
    manual_mode_setpoint: float
    heating_operation_mode: str
    heating_state: str
    humidity: float
    set_back_temperature: float
    time_windows: dict


class Circuit(BaseModel):
    system_id: str
    index: int
    circuit_state: str
    current_circuit_flow_temperature: float
    heating_curve: float
    is_cooling_allowed: bool
    min_flow_temperature_setpoint: float
    mixer_circuit_type_external: str
    set_back_mode_enabled: bool
    zones: List = []


class DomesticHotWater(BaseModel):
    system_id: str
    index: int
    current_dhw_tank_temperature: float
    current_special_function: str
    max_set_point: float
    min_set_point: float
    operation_mode: str
    set_point: float
    time_windows: dict


class System(BaseModel):
    id: str
    status: Dict[str, bool]
    devices: List[Dict]
    current_system: Dict = {}
    system_configuration: Dict = {}
    system_control_state: Dict = {}
    gateway: Dict = {}
    has_ownership: bool
    zones: List[Zone] = []
    circuits: List[Circuit] = []
    domestic_hot_water: List[DomesticHotWater] = []

    def __init__(self, **data: Any) -> None:
        super().__init__(**data)
        zones = self.system_control_state["control_state"]["zones"]
        circuits = self.system_control_state["control_state"]["circuits"]
        domestic_hot_water = self.system_control_state["control_state"][
            "domestic_hot_water"
        ]
        self.zones = [Zone(system_id=self.id, **z) for z in zones]
        self.circuits = [Circuit(system_id=self.id, **c) for c in circuits]
        self.domestic_hot_water = [
            DomesticHotWater(system_id=self.id, **d) for d in domestic_hot_water
        ]

    @property
    def outdoor_temperature(self):
        return self.system_control_state["control_state"]["general"][
            "outdoor_temperature"
        ]

    @property
    def water_pressure(self):
        return self.system_control_state["control_state"]["general"][
            "system_water_pressure"
        ]

    @property
    def mode(self):
        return self.system_control_state["control_state"]["general"]["system_mode"]


class Device(BaseModel):
    system: System
    device_uuid: str
    name: str = ""
    product_name: str
    diagnostic_trouble_codes: List = []
    properties: List = []
    ebus_id: str
    article_number: str
    device_serial_number: str
    device_type: str
    first_data: datetime.datetime
    last_data: datetime.datetime
    operational_data: dict = {}
    data: list = []



import re
import string
import random
import hashlib
import base64


snake_pattern = re.compile(r"(?<!^)(?=[A-Z])")


def to_snake(s):
    return snake_pattern.sub("_", s).lower()


def dict_to_snake_case(d):
    if isinstance(d, list):
        return [dict_to_snake_case(i) if isinstance(i, (dict, list)) else i for i in d]
    return {
        to_snake(a): dict_to_snake_case(b) if isinstance(b, (dict, list)) else b
        for a, b in d.items()
    }


def random_string(length):
    return "".join(
        random.choices(
            string.ascii_lowercase + string.ascii_uppercase + string.digits, k=length
        )
    )


def generate_code() -> tuple[str, str]:
    rand = random.SystemRandom()
    code_verifier = "".join(rand.choices(string.ascii_letters + string.digits, k=128))

    code_sha_256 = hashlib.sha256(code_verifier.encode("utf-8")).digest()
    b64 = base64.urlsafe_b64encode(code_sha_256)
    code_challenge = b64.decode("utf-8").replace("=", "")

    return (code_verifier, code_challenge)



import datetime
from urllib.parse import urlencode

from myPyllant.session import get_session, get_authorized_headers
from myPyllant.models import System, Circuit, Device, DomesticHotWater, Zone
from myPyllant.utils import dict_to_snake_case


base_api_url = (
    "https://api.vaillant-group.com/service-connected-control/end-user-app-api/v1"
)


async def get_systems():
    session = get_session()
    systems_url = f"{base_api_url}/systems"
    async with session.get(
        systems_url, headers=get_authorized_headers()
    ) as systems_resp:
        for system_json in await systems_resp.json():
            system_id = system_json["systemId"]
            system_url = f"{base_api_url}/emf/v2/{system_id}/currentSystem"

            async with session.get(
                system_url, headers=get_authorized_headers()
            ) as current_system_resp:
                current_system = await current_system_resp.json()
            system = System(
                id=system_id,
                current_system=current_system,
                **dict_to_snake_case(system_json),
            )
            yield system


async def get_devices_by_system(system: System, get_buckets: bool = False):
    session = get_session()
    for device_raw in system.current_system.values():
        if not (isinstance(device_raw, dict) and "device_uuid" in device_raw):
            continue
        device = Device(
            system=system,
            **device_raw,
        )

        serial_nos = {d["serial_number"]: d for d in system.devices}

        if device.device_serial_number in serial_nos.keys():
            device_info = serial_nos[device.device_serial_number]
            device.operational_data = dict_to_snake_case(
                device_info.get("operational_data", {})
            )
            device.name = device_info.get("name", "")

        if get_buckets:
            for data in device.data:
                querystring = {
                    "resolution": "DAY",
                    "operationMode": data["operation_mode"],
                    "energyType": data["value_type"],
                    "startDate": data["from"],
                    "endDate": data["to"],
                }
                device_buckets_url = f"{base_api_url}/emf/v2/{system.id}/devices/{device.device_uuid}/buckets?{urlencode(querystring)}"
                async with session.get(
                    device_buckets_url, headers=get_authorized_headers()
                ) as device_buckets_resp:
                    data["buckets"] = dict_to_snake_case(
                        (await device_buckets_resp.json())["data"]
                    )
        yield device


async def quick_veto_zone_temperature(
    zone: Zone, temperature: float, duration_hours: int
):
    session = get_session()
    url = f"{base_api_url}/systems/{zone.system_id}/zones/{zone.index}/quickVeto"
    return await session.patch(
        url,
        json={
            "desiredRoomTemperatureSetpoint": temperature,
            "duration": duration_hours,
        },
        headers=get_authorized_headers(),
    )


async def cancel_quick_veto_zone_temperature(zone: Zone):
    session = get_session()
    url = f"{base_api_url}/systems/{zone.system_id}/zones/{zone.index}/quickVeto"
    return await session.delete(url, headers=get_authorized_headers())


async def set_set_back_temperature(zone: Zone, temperature: float):
    session = get_session()
    url = (
        f"{base_api_url}/systems/{zone.system_id}/zones/{zone.index}/setBackTemperature"
    )
    return await session.patch(
        url, json={"setBackTemperature": temperature}, headers=get_authorized_headers()
    )


async def set_holiday(
    system: System, start: datetime.datetime, end: datetime.datetime = None
):
    if not end:
        # Set away for a long time if no end date is set
        end = start + datetime.timedelta(days=365)
    assert start < end
    session = get_session()
    url = f"{base_api_url}/systems/{system.id}/holiday"
    data = {
        "holidayStartDateTime": start.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z",
        "holidayEndDateTime": end.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z",
    }
    return await session.post(url, json=data, headers=get_authorized_headers())


async def cancel_holiday(system: System):
    session = get_session()
    url = f"{base_api_url}/systems/{system.id}/holiday"
    return await session.delete(url, headers=get_authorized_headers())


async def set_domestic_hot_water_temperature(
    domestic_hot_water: DomesticHotWater, temperature: int
):
    session = get_session()
    url = f"{base_api_url}/systems/{domestic_hot_water.system_id}/domesticHotWater/{domestic_hot_water.index}/temperature"
    return await session.post(
        url, json={"setPoint": temperature}, headers=get_authorized_headers()
    )


async def boost_domestic_hot_water(domestic_hot_water: DomesticHotWater):
    session = get_session()
    url = f"{base_api_url}/systems/{domestic_hot_water.system_id}/domesticHotWater/{domestic_hot_water.index}/boost"
    return await session.post(url, json={}, headers=get_authorized_headers())


async def cancel_hot_water_boost(domestic_hot_water: DomesticHotWater):
    session = get_session()
    url = f"{base_api_url}/systems/{domestic_hot_water.system_id}/domesticHotWater/{domestic_hot_water.index}/boost"
    return await session.delete(url, headers=get_authorized_headers())
